/************************************************************************************
 * Le sketch qui commande le four à refusion pour la soudure des composants CMS par *
 * refusion de pâte à braser ... Mesure de la température par thermocouple et       * 
 * MAX 31855 aux paliers et temps requis . Indication de la température et du danger*
 * "four chaud" ... Un bouton de démarrage de cycle et arrêt jusqu'au POR ou Reset  *
 * avant un nouveau départ ... B.LECLERC https://amazing-vacuum-tubes.com           *
 ************************************************************************************/

 //Appel des librarys utilisées et déclaration des Entrées / Sorties
 
#include <LiquidCrystal.h>
#include <Wire.h>

const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

#include <SPI.h>
#include "Adafruit_MAX31855.h"

#define MAXDO   8
#define MAXCS   9
#define MAXCLK  10

#define REL A1
#define LED A0

#define PBUT A2

double Tmp;
int Time = 0;
bool cyclOK = 0;

Adafruit_MAX31855 thermocouple(MAXCLK, MAXCS, MAXDO);


void setup() { //Prerequis du démarrage ...

  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.write("amazing-vacuum-");
  lcd.setCursor(0, 1);
  lcd.write("tubes.com");
  delay(2000);

  while (!Serial) delay(1);
  delay(500);
  if (!thermocouple.begin()) {
    while (1) delay(10);
  }

// Direction des entrées /sorties
  pinMode(REL, OUTPUT);
  digitalWrite(REL, LOW);
  pinMode(LED, OUTPUT);
  digitalWrite(REL, LOW);
  pinMode(PBUT, INPUT_PULLUP);
}

void loop() {

  double Tmp = thermocouple.readCelsius();//lecture du thermocouple 

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.write("Temperature:");
  lcd.print(Tmp);

  if (cyclOK == 0) { //Attente du départ du cycle 
  lcd.setCursor(0, 1);
  lcd.write("Push to start !");
  }


  if (Tmp > 55) {
    digitalWrite(LED, HIGH);
  }

  if (digitalRead(PBUT) == LOW) {// detection d'appui sur le bouton pour départ de cycle 
    cyclOK = 1;
    digitalWrite(REL, HIGH);
  }
  delay(1000);

  if ((Tmp >= 120) and (cyclOK == 1))  { //Si la température est atteinte saut au premier palier  
    digitalWrite(REL, LOW);
    Time = 90;
    prechauf();
  }
}

void prechauf() { // gestion du premier palier 120° pendant 1'30"
  double Tmp = thermocouple.readCelsius();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.write("Temperature:");
  lcd.print(Tmp);
  lcd.setCursor(0, 1);
  lcd.write("Time :");
  lcd.print(Time);

  if (Tmp >= 120) { //Régulation ....
    digitalWrite(REL, LOW);
  } else {
    digitalWrite(REL, HIGH);
  }
  delay(1000);

  if (Time > 1 ) { //Temporisation
    Time--;
    prechauf();
  } else {
     digitalWrite(REL, HIGH);    
       nxTwo();        
  }
}

void nxTwo()  {  //montée à 190°
  double Tmp = thermocouple.readCelsius();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.write("Temperature:");
  lcd.print(Tmp);
  delay(1000);
  if (Tmp > 190) {
    Time = 45;
      reflow();    
  } else {
      nxTwo();
  }
}


void reflow() { //Le soudage : 190° pendant 45"
  double Tmp = thermocouple.readCelsius();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.write("Temperature:");
  lcd.print(Tmp);
  lcd.setCursor(0, 1);
  lcd.write("Time :");
  lcd.print(Time);

  if (Tmp >= 190) {  // Régulation
    digitalWrite(REL, LOW);
  } else {
    digitalWrite(REL, HIGH);
  }
  delay(1000);

  if (Time > 1 ) { //Temporisation
    Time--;
    reflow();
  } else {
    cyclOK == 0;
    digitalWrite(REL, LOW);
    finProg();
  }
}

void finProg() {  // Refroidissement
  double Tmp = thermocouple.readCelsius();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.write("Temperature:");
  lcd.print(Tmp);
  lcd.setCursor(0, 1);
  lcd.write("Attention => Hot");
  delay (3000);
  if (Tmp < 60) {
    digitalWrite(LED, LOW);
      lcd.clear();
    lcd.write("      FINI !   ");
    delay (1000);      
        fini();
  }
  finProg(); 
}

void fini() { // Fin de cycle 
      delay (10000);
      fini();
  
}
